@@ -30,6 +30,7 @@ import sys
 import pwd
 import errno
 import signal
+import select
 import time
 import resource
 import configparser
@@ -66,7 +67,7 @@ UMASK = 0o0022
 MAXFD = 1024
 REDIRECT_TO = "/dev/null"
 # Default Command line help messages
-STUB_USAGE_MESSAGE = "Usage: %s [-dhv] [-c config_file] [-r rpdb2_wait] (Stub 'usage_short()' msg)"
+STUB_USAGE_MESSAGE = "Usage: %s [-dhv] [-c config_file] (Stub 'usage_short()' msg)"
 STUB_COMMAND_DESCRIPTION = "Command description (Stub 'usage_full()' message.)"
 
 
@@ -292,8 +293,6 @@ class DebugCmdLineArg(BaseCmdLineArg):
 DEFAULT_CMDLINE_ARG_LIST = [ConfigCmdLineArg(),
                             DebugCmdLineArg(),
                             HelpCmdLineArg(),
-                            MemoryDebugCmdLineArg(),
-                            SystemdCmdLineArg(),
                             VerboseCmdLineArg()]
 
 
@@ -449,7 +448,7 @@ class Process(ProcessCmdLine):
         """
         Check that we are running as root, if not exit with message.
 
-	ProcessDaemon bleow overrides this.  It is here for script utilities.
+	ProcessDaemon below overrides this.  It is here for script utilities.
         """
         # check that we are root for file writing permissions stuff
         if (os.geteuid() != 0 ):
@@ -523,7 +522,7 @@ class DaemonOperations(object):
             # Sending signal 0 does not touch process, but  call succeeds
             # if it exists
             os.kill(old_pid, 0)
-            if (self.i_am_daemon()):
+            if (old_pid == os.getpid()):
                 return True
         except ValueError as e:
             # Error from int() type conversion above
@@ -626,17 +625,20 @@ class DaemonOperations(object):
         maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
         if (maxfd == resource.RLIM_INFINITY):
             maxfd = MAXFD
-        
-        # Iterate through and close all file descriptors.
-        for fd in range(0, maxfd):
-            if (fd_top):
-                # skip python logging/syslog fds etc!
-                if (fd <= fd_top and fd > 2):
-                    continue
-            try:
-                os.close(fd)
-            except (IOError, OSError):     # ERROR, fd wasn't open to begin with (ignored)
-                pass
+
+        # Iterate through and close all file descriptors
+        # Don't try and close file descriptors twice on SIGHUP as per 
+        # man 2 close(), process observed getting reaped somehow...
+        if (not self.i_am_daemon()):
+            for fd in range(0, maxfd):
+                if (fd_top):
+                    # skip python logging/syslog fds etc!
+                    if (fd <= fd_top and fd > 2):
+                        continue
+                try:
+                    os.close(fd)
+                except (IOError, OSError):     # ERROR, fd wasn't open to begin with (ignored)
+                    pass
 
         # Redirect stderr, stdout stdin
         # This call to open is guaranteed to return the lowest file 
@@ -755,7 +757,7 @@ class DaemonOperations(object):
             os.chdir(WORKDIR)
             os.umask(UMASK)
             self._reduce_privilege()
-            # create a PID file since we are running in debug mode
+            # create a PID file since we are running in systemd mode
             self.create_pid_file()
             # Exit function
             return
@@ -796,11 +798,52 @@ class DaemonOperations(object):
         pgid = os.getpgid(0)
         ppid = os.getppid()
 
-        if (ppid == 1  and sid == pgid and pid != sid):
+        # Being detached from a controlling terminal is the major detectable
+        # outcome of legacy daemonisation.  Only use this one major property
+        # as the determinant for this test.  Systemd may mess with program 
+        # group/session IDs as it 'keeps'/moves the process back to a 
+        # controlling session for a user????
+        if (not os.isatty(sys.stdin.fileno())):
             return True
         else:
             return False
     
+class MainSleep(object):
+    """
+    Interruptable sleep() class for processes main()
+
+    Alternative to Python sleep.time(), and IS interruptible with signals
+    After Python 3.5 time.sleep() is effectively non-interruptable with signals
+    """
+
+    def init_sleep(self):
+        """
+        Initialises sleep functionality by set up named pipe and setting
+        up signal delivery via this
+        """
+        self._fdpipe = os.pipe2(os.O_NONBLOCK|os.O_CLOEXEC)
+        signal.set_wakeup_fd(self._fdpipe[1])
+
+    def main_sleep(self, seconds):
+        """
+        Interruptable sleep() function for processes main()
+        """
+        secs = seconds
+        if (type(secs) is str):
+            secs = int(secs.strip())
+        elif (type(secs) is not int):
+            secs = int(secs)
+        while True:
+            try:
+                nr, blah1, blah2 = select.select([self._fdpipe[0]], [], [], secs)
+                if nr:
+                    os.read(self._fdpipe[0], 16384)
+                break
+            except (IOError, OSError) as exc:
+                if exc.errno not in (errno.EINTR, errno.EAGAIN):
+                    raise(exc)
+        return
+
 
 class SignalHandler(object):
     """
@@ -941,7 +984,7 @@ class SignalBusiness(object):
 
 
 
-class ProcessDaemon(Process, DaemonOperations, SignalBusiness):
+class ProcessDaemon(Process, DaemonOperations, SignalBusiness, MainSleep):
     """
     Main daemon process class
     """
@@ -951,7 +994,9 @@ class ProcessDaemon(Process, DaemonOperations, SignalBusiness):
         Add wait for Rpdb2 to commande line
         """
         Process.__init__(self, *args, **kwargs)
+        self.cmdline_arg_list.append(MemoryDebugCmdLineArg())
         self.cmdline_arg_list.append(Rpdb2WaitCmdLineArg())
+        self.cmdline_arg_list.append(SystemdCmdLineArg())
     
     def check_if_root(self):
         """
@@ -999,7 +1044,7 @@ class ProcessDaemon(Process, DaemonOperations, SignalBusiness):
             sleep_time = float(settings['sleep_time'])
             log_debug("Process.main_process() - sleep(%s) seconds."
                     % sleep_time) 
-            time.sleep(sleep_time)
+            self.main_sleep(sleep_time)
 
         log_info('Exited main loop - process terminating normally.')
         sys.exit(os.EX_OK)
@@ -1018,6 +1063,7 @@ class ProcessDaemon(Process, DaemonOperations, SignalBusiness):
         self.daemonize()
         setup_file_logging()
         self.init_signals()
+        self.init_sleep()
         self.main_process()
 
 
